package org.processmining.stagedprocessflows.plugins;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.joda.time.DateTime;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.stagedprocessflows.models.SPF;
import org.processmining.stagedprocessflows.models.SPFManager;
import org.processmining.stagedprocessflows.parameters.SPFConfig;
import org.processmining.stagedprocessflows.ui.main.SPFFrame;
import org.processmining.stagedprocessflows.ui.wizard.WizardUI;
import org.processmining.stagedprocessflows.util.LogUtilites;

/**
 * This is the main plugin to be installed in PROM
 * 
 * @author Hoang Nguyen
 * @date 21 Oct 2015
 * @since 21 Oct 2015
 * @version 1.0
 * @email huanghuy.nguyen@hdr.qut.edu.au
 * @copyright Hoang Nguyen BPM Discipline Queensland University of Technology
 */

@Plugin(name = "Performance Mining With Staged Process Flows", parameterLabels = { "Log" }, returnLabels = { "Performance Mining With Staged Process Flows" }, returnTypes = { SPFFrame.class }, userAccessible = true)
public class SPFPlugin {

	//@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Hoang Nguyen", email = "huanghuy.nguyen@hdr.qut.edu.au")
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Hoang Nguyen", email = "huanghuy.nguyen@hdr.qut.edu.au")
	@PluginVariant(variantLabel = "", requiredParameterLabels = { 0 })
	public static SPFFrame create(UIPluginContext context, final XLog log) {

		SPFManager.getInstance().clear(); //start over a new BPF
		SPFConfig config = new SPFConfig();
		readData(log, config, SPFManager.getInstance());
		config.setContext(context);
		WizardUI bpfUI = new WizardUI(context, config);
		SPFFrame frame = bpfUI.mine(log);

		return frame;
	}

	//	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Busienss Process Flow", email = "")
	//	@PluginVariant(variantLabel = "", requiredParameterLabels = { 0 })
	//	public static BPFFrame create(PluginContext context, final XLog log, BpfConfig config) {
	//		//Update existing configuration
	//		
	//		//Call
	//		WizardUI bpfUI = new WizardUI(context, config);
	//		BPFFrame frame = bpfUI.mine(log);
	//		
	//		return frame;
	//	}	

	/**
	 * This method reads the log file and populate data to the Config and
	 * BPFManager object
	 * 
	 * @param log
	 * @param config
	 * @param bpfManager
	 */
	private static void readData(XLog log, SPFConfig config, SPFManager bpfManager) {
		new HashMap<String, Map<String, String>>();
		config.setXLog(log);

		for (XTrace trace : log) {
			DateTime traceStart = new DateTime(9999, 12, 1, 1, 0);
			DateTime traceEnd = new DateTime(0);

			//---------------------------------------
			// Populate the Config object
			//---------------------------------------
			for (XEvent event : trace) {
				String eventName = LogUtilites.getConceptName(event).toLowerCase();
				String stage = LogUtilites.getValue(event.getAttributes().get("stage")).toLowerCase();
				//					String type = LogUtilites.getValue(event.getAttributes().get("type")).toLowerCase();
				String transitiontype = LogUtilites.getLifecycleTransition(event).toLowerCase();
				DateTime eventTime = LogUtilites.getTimestamp(event);

				if (!config.getStageList().contains(stage)) {
					config.getStageList().add(stage);
				}

				if (!config.getEventStageMap().containsKey(eventName)) {
					config.getEventStageMap().put(eventName, stage);
				}

				//					if (type.equals("activity")) {
				//						config.getActivityList().add(eventName);
				//					}
				//					else if (type.equals("gate")) {
				//						config.getGateList().add(eventName);
				//					}

				if (traceStart.isAfter(eventTime)
						&& (transitiontype.equals("start") || transitiontype.equals("complete"))) {
					traceStart = eventTime;
				}

				if (traceEnd.isBefore(eventTime)
						&& (transitiontype.equals("start") || transitiontype.equals("complete"))) {
					traceEnd = eventTime;
				}
			}

			String traceStatus = LogUtilites.getValue(trace.getAttributes().get("status")).toLowerCase();
			if (!config.getCaseStatusList().contains(traceStatus)) {
				config.getCaseStatusList().add(traceStatus);
			}

			//---------------------------------------
			// Populate the BPFManager object
			//---------------------------------------
			String curTraceID = LogUtilites.getConceptName(trace);
			Map<String, String> casePropertyMap = new HashMap<String, String>();
			Iterator<XAttribute> attIterator = trace.getAttributes().values().iterator();

			while (attIterator.hasNext()) {
				XAttribute att = attIterator.next();
				if (!att.getKey().equals("concept:name")) {
					casePropertyMap.put(att.getKey(), LogUtilites.getValue(att));
				}
			}

			casePropertyMap.put(SPF.CASE_START_TIME, String.valueOf(traceStart.getMillis()));
			casePropertyMap.put(SPF.CASE_END_TIME, String.valueOf(traceEnd.getMillis()));
			casePropertyMap.put(SPF.CASE_EVENT_ACOUNT, String.valueOf(trace.size()));

			bpfManager.getCaseAttributeMap().put(curTraceID, casePropertyMap);
		}
	}

}
