package org.processmining.stagedprocessflows.ui.service;

import org.jfree.chart.labels.XYToolTipGenerator;
import org.jfree.data.time.TimeTableXYDataset;
import org.jfree.data.xy.XYDataset;
import org.joda.time.DateTimeZone;
import org.processmining.stagedprocessflows.models.FlowCell;
import org.processmining.stagedprocessflows.models.SPF;
import org.processmining.stagedprocessflows.models.Stage;

public class SPFDepartureViewTooltipGenerator implements XYToolTipGenerator {
	private SPF bpf = null;

	//	Map<TimePeriod,DateTime> timeMap;

	public SPFDepartureViewTooltipGenerator(SPF bpf) {
		this.bpf = bpf;
		//		this.timeMap = timeMap;
	}

	public String generateToolTip(XYDataset dataset, int series, int item) {
		StringBuilder stringBuilder = new StringBuilder();
		String seriesName = dataset.getSeriesKey(series).toString();
		stringBuilder.append(String.format("<html><p style='color:#0000ff;'>Stage: '%s'</p>", seriesName));

		((TimeTableXYDataset) dataset).getTimePeriod(item);
		String stageName = getStageName(seriesName);
		Stage stage = bpf.getStageByName(stageName);
		FlowCell cell = stage.getFlowCells().get(item);

		if (cell != null) {
			stringBuilder.append(bpf.getTimeSeries().get(item)
					.withZone(DateTimeZone.forTimeZone(bpf.getConfig().getTimeZone()))
					.toString("EEE, d MMM yyyy HH:mm:ss Z")
					+ "<br/>");

			for (String exitType : stage.getServiceExitSubCounts().keySet()) {
				stringBuilder.append(exitType + "(cases): " + stage.getServiceExitSubCounts().get(exitType).get(item)
						+ "<br/>");
				stringBuilder.append(exitType + "_rate(cases/h): "
						+ cell.getCharacteristic(SPF.CHAR_SERVICE_EXIT_RATE_TYPE + exitType) + "<br/>");
			}

			int passedCount = bpf.getStageByName(stageName).getServicePassedCounts().get(item);
			stringBuilder.append("Passed(cases): " + passedCount + "<br/>");
			stringBuilder.append("PR(cases/h): " + cell.getCharacteristic(SPF.CHAR_SERVICE_PASSED_RATE));
		}
		stringBuilder.append("</html>");
		return stringBuilder.toString();
	}

	/**
	 * Extract the stage name from the series name The series name must have
	 * format: <StageName>-<Suffix>
	 * 
	 * @param seriesName
	 * @return the stage name. In case the series name is also the stage name
	 *         (without "-") then return the whole series name.
	 */
	private String getStageName(String seriesName) {
		if (seriesName.lastIndexOf("-") != -1) {
			return seriesName.substring(0, seriesName.lastIndexOf("-"));
		} else {
			return seriesName;
		}
	}

}
